package com.javacodegeeks.nio.heapbytebuffer;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.nio.ByteBuffer;
import java.nio.ReadOnlyBufferException;
import java.util.function.Consumer;

import org.junit.Before;
import org.junit.Test;

public class HeapByteBufferReadOnlyTest {
    private ByteBuffer buffer;

    @Before
    public void setUp() {
        this.buffer = ByteBuffer.allocate(5 * 10000);
    }

    @Test(expected = ReadOnlyBufferException.class)
    public void readOnly() {
        this.buffer = this.buffer.asReadOnlyBuffer();
        assertTrue("Must be readOnly", this.buffer.isReadOnly());
        this.buffer.putChar('b');
    }

    @Test(expected = ReadOnlyBufferException.class)
    public void readOnlyWithManners() {
        this.buffer = this.buffer.asReadOnlyBuffer();
        assertTrue("Must be readOnly", this.buffer.isReadOnly());
        this.buffer.put("Please put this in the buffer".getBytes());
    }

    @Test
    public void persistenReadOnlyChecker() {
        this.buffer = buffer.asReadOnlyBuffer();

        check(this.buffer, (a) -> a.duplicate().put("I am going to trick this buffer".getBytes()));
        check(this.buffer, (a) -> a.slice().put("I am going to trick this buffer".getBytes()));
        check(this.buffer, (a) -> a.put("I am going to trick this buffer".getBytes()));
        check(this.buffer, (a) -> a.asIntBuffer().put(1));
        check(this.buffer, (a) -> a.asCharBuffer().put('c'));
        check(this.buffer, (a) -> a.asFloatBuffer().put(1f));
    }

    private void check(final ByteBuffer buffer, final Consumer<? super ByteBuffer> action) {
        try {
            action.accept(buffer);
            fail("Must throw exception");
        } catch (ReadOnlyBufferException e) {
        }
    }
}
